package dev.pfaff.altargui.widgets;

import dev.pfaff.altargui.*;
import org.jetbrains.annotations.NotNull;

/**
 * @param aspectRatio the desired aspect ratio, in the form {@code width / height}.
 */
public record AspectRatioWidget(Widget child, double aspectRatio) implements SingleChildWidget {
	@Override
	public @NotNull WidgetInstance<? extends SingleChildWidget> instantiate(WidgetInstantiationArgs args) {
		return new State(args);
	}

	private static final class State extends CustomLayoutState<AspectRatioWidget> {
		private double aspectRatio;

		private State(WidgetInstantiationArgs args) {
			super(args);
		}

		@Override
		public void configure(StateContext stateContext, AspectRatioWidget widget, boolean init) {
			super.configure(stateContext, widget, init);
			if (this.aspectRatio != widget.aspectRatio) {
				this.aspectRatio = widget.aspectRatio;
				this.markNeedsStencilLayout();
			}
		}

		@Override
		protected void computeStencilLayout(StencilLayoutContext context) {
			var allocWidth = allocatedWidth();
			var allocHeight = allocatedHeight();
			var aspectRatio = this.aspectRatio;

			float width, height;
			var aspectRatioWidthFromHeight = (float) (allocHeight * aspectRatio);
			if (aspectRatioWidthFromHeight <= allocWidth) {
				var aspectRatioHeightFromWidth = (float) (allocWidth / aspectRatio);
				assert aspectRatioHeightFromWidth <= allocHeight : STR."the math is not mathing: allocatedWidth=\{allocWidth}, allocatedHeight=\{allocHeight}, aspectRatio=\{aspectRatio}";
				width = allocWidth;
				height = aspectRatioHeightFromWidth;
			} else {
				width = aspectRatioWidthFromHeight;
				height = allocHeight;
			}

			context.layout(this, getChildElement(), (e, ctx) -> {
				e.setAvailableSize(width, height);
				e.stencilLayout(ctx);
				copyComputedSizeFrom(e);
			});
		}

		@Override
		protected void computeTraceLayout() {
			var e = getChildElement();
			e.copyPositionFrom(this);
			e.traceLayout();
		}
	}
}
